function [S,C] = score_chord_id(T,R,isharte)
% [S,C] = score_chord_id(T,R,isharte)
%     Score chord recognition systems.  
%     T is the test data (system output), R is the reference
%     (truth) data.  
%     T and R can be any of:
%      - single two-column arrays with the first column the start
%        time (in secs) and the second col the label (e.g. 0..24)
%      - a string containing the name of a file in this format
%      - a cell array consisting of multiple file name strings,
%        paired between T and R.
%     S returns as the overall accuracy (between 0 and 1); C
%     returns a confusion matrix (e.g. 25 x 25)
%     <isharte> nonzero means data files are <start> <end>
%     <asciilabel> instead of <start> <chord0..24>
% 2008-08-24 Dan Ellis dpwe@ee.columbia.edu

if nargin < 3
  isharte = 0;
end

% Normalize input formats
% If they are single filenames (or data matrices), 
% make them one-element cell arrays
if ~iscell(T)  tmp = T;  T = cell(1,1);  T{1} = tmp;  end
if ~iscell(R)  tmp = R;  R = cell(1,1);  R{1} = tmp;  end

C = zeros(25,25);  % I guess - hard coded for 25-state chord contest

for i = 1:length(T)
  TT = T{i};
  RR = R{i};
  % If they are filenames, just read in the data
  if isharte == 0
    if isstr(TT)   TT = textread(TT);  end
    if isstr(RR)   RR = textread(RR);  end
  else
    if isstr(TT)   TT = readharte(TT);  end
    if isstr(RR)   RR = readharte(RR);  end
  end
  
    

  % Now the main routine: TT and RR are two <start time> <label> 
  % streams.  Calculate their total times in each pair of states
  alltimes = unique(sort([TT(:,1)',RR(:,1)']));
  
  % We ignore the final symbol since we don't have an end time for it
  for j = 1:length(alltimes)-1
    thistime = alltimes(j);
    nexttime = alltimes(j+1);
    segdur = nexttime - thistime;
    Tsym = lookup(TT,thistime);
    Rsym = lookup(RR,thistime);

    % update confusion matrix with this amount of time
    C(1+Rsym,1+Tsym) = C(1+Rsym,1+Tsym) + segdur;
  
  end
end

% Actual accuracy %
S = sum(diag(C)) / sum(C(:));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function L = lookup(D,T)
% L = lookup(D,T)
%    D is a set of rows <starttime> <label>
%    Find the interval D(i,1) <= T && D(i+1,1) > T
%    and return L = D(i,2)
% 2008-08-24 Dan Ellis dpwe@ee.columbia.edu

% Make sure there is a zero entry, but return special symbol if it's used
% (will be superceded by a real zero entry)

nochordsymbol = 24;  %%%%%%% SPECIAL CASE ********

D = [[0,nochordsymbol];D];

L = D(max(find(D(:,1)<=T)),2);

